# coding: utf-8

# flake8: noqa
"""
    allure-testops-service

    Branch **HEAD**  Commit **623f6ed302ba4b651cf9040faca4635af2d93b7c**

    The version of the OpenAPI document: 25.4.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

# import models into model package
from src.client.generated.models.access_group_bulk_dto import AccessGroupBulkDto
from src.client.generated.models.access_group_create_dto import AccessGroupCreateDto
from src.client.generated.models.access_group_dto import AccessGroupDto
from src.client.generated.models.access_group_pa_dto import AccessGroupPaDto
from src.client.generated.models.access_group_patch_dto import AccessGroupPatchDto
from src.client.generated.models.access_group_project_access_dto import AccessGroupProjectAccessDto
from src.client.generated.models.access_group_projects_add_dto import AccessGroupProjectsAddDto
from src.client.generated.models.access_group_user_dto import AccessGroupUserDto
from src.client.generated.models.access_group_users_add_dto import AccessGroupUsersAddDto
from src.client.generated.models.allowed_role_dto import AllowedRoleDto
from src.client.generated.models.analytic_automation_trend_dto import AnalyticAutomationTrendDto
from src.client.generated.models.analytic_dto import AnalyticDto
from src.client.generated.models.analytic_interval import AnalyticInterval
from src.client.generated.models.analytic_launch_duration_histogram_dto import AnalyticLaunchDurationHistogramDto
from src.client.generated.models.analytic_mute_trend_dto import AnalyticMuteTrendDto
from src.client.generated.models.analytic_tc_automation_count_dto import AnalyticTcAutomationCountDto
from src.client.generated.models.analytic_tc_status_count_dto import AnalyticTcStatusCountDto
from src.client.generated.models.analytic_tr_by_status_trend_dto import AnalyticTrByStatusTrendDto
from src.client.generated.models.aql_validate_response_dto import AqlValidateResponseDto
from src.client.generated.models.assign_request_dto import AssignRequestDto
from src.client.generated.models.async_job_dto import AsyncJobDto
from src.client.generated.models.async_job_status_dto import AsyncJobStatusDto
from src.client.generated.models.async_job_step_dto import AsyncJobStepDto
from src.client.generated.models.async_job_step_status_dto import AsyncJobStepStatusDto
from src.client.generated.models.async_job_with_steps_dto import AsyncJobWithStepsDto
from src.client.generated.models.attachment_row import AttachmentRow
from src.client.generated.models.attachment_step_dto import AttachmentStepDto
from src.client.generated.models.attrs import Attrs
from src.client.generated.models.audit_action_type_dto import AuditActionTypeDto
from src.client.generated.models.blob_storage_stats import BlobStorageStats
from src.client.generated.models.blob_storage_update_stats import BlobStorageUpdateStats
from src.client.generated.models.body_step_dto import BodyStepDto
from src.client.generated.models.bold_mark import BoldMark
from src.client.generated.models.business_metric_dto import BusinessMetricDto
from src.client.generated.models.category_create_dto import CategoryCreateDto
from src.client.generated.models.category_dto import CategoryDto
from src.client.generated.models.category_matcher_create_dto import CategoryMatcherCreateDto
from src.client.generated.models.category_matcher_dto import CategoryMatcherDto
from src.client.generated.models.category_matcher_patch_dto import CategoryMatcherPatchDto
from src.client.generated.models.category_patch_dto import CategoryPatchDto
from src.client.generated.models.cfv_merge_to_existing_global_value_dto import CfvMergeToExistingGlobalValueDto
from src.client.generated.models.cfv_merge_to_new_global_value_dto import CfvMergeToNewGlobalValueDto
from src.client.generated.models.ci import Ci
from src.client.generated.models.cleaner_schema_create_dto import CleanerSchemaCreateDto
from src.client.generated.models.cleaner_schema_dto import CleanerSchemaDto
from src.client.generated.models.cleaner_schema_patch_dto import CleanerSchemaPatchDto
from src.client.generated.models.cleaner_schema_target_dto import CleanerSchemaTargetDto
from src.client.generated.models.code_block_document_node import CodeBlockDocumentNode
from src.client.generated.models.code_mark import CodeMark
from src.client.generated.models.comment_create_dto import CommentCreateDto
from src.client.generated.models.comment_dto import CommentDto
from src.client.generated.models.comment_patch_dto import CommentPatchDto
from src.client.generated.models.create_launch_event import CreateLaunchEvent
from src.client.generated.models.csv_import_options import CsvImportOptions
from src.client.generated.models.custom_field_bulk_add_to_projects_dto import CustomFieldBulkAddToProjectsDto
from src.client.generated.models.custom_field_bulk_archive_dto import CustomFieldBulkArchiveDto
from src.client.generated.models.custom_field_create_dto import CustomFieldCreateDto
from src.client.generated.models.custom_field_dto import CustomFieldDto
from src.client.generated.models.custom_field_export_field_mapping import CustomFieldExportFieldMapping
from src.client.generated.models.custom_field_import_field_mapping import CustomFieldImportFieldMapping
from src.client.generated.models.custom_field_merge_dto import CustomFieldMergeDto
from src.client.generated.models.custom_field_patch_dto import CustomFieldPatchDto
from src.client.generated.models.custom_field_project_count_dto import CustomFieldProjectCountDto
from src.client.generated.models.custom_field_project_dto import CustomFieldProjectDto
from src.client.generated.models.custom_field_project_patch_dto import CustomFieldProjectPatchDto
from src.client.generated.models.custom_field_project_with_values_dto import CustomFieldProjectWithValuesDto
from src.client.generated.models.custom_field_raw_dto import CustomFieldRawDto
from src.client.generated.models.custom_field_schema_count_dto import CustomFieldSchemaCountDto
from src.client.generated.models.custom_field_schema_create_dto import CustomFieldSchemaCreateDto
from src.client.generated.models.custom_field_schema_dto import CustomFieldSchemaDto
from src.client.generated.models.custom_field_schema_patch_dto import CustomFieldSchemaPatchDto
from src.client.generated.models.custom_field_value_bulk_delete_dto import CustomFieldValueBulkDeleteDto
from src.client.generated.models.custom_field_value_bulk_merge_dto import CustomFieldValueBulkMergeDto
from src.client.generated.models.custom_field_value_dto import CustomFieldValueDto
from src.client.generated.models.custom_field_value_patch_dto import CustomFieldValuePatchDto
from src.client.generated.models.custom_field_value_project_create_dto import CustomFieldValueProjectCreateDto
from src.client.generated.models.custom_field_value_project_merge_by_id_dto import CustomFieldValueProjectMergeByIdDto
from src.client.generated.models.custom_field_value_project_merge_by_name_dto import CustomFieldValueProjectMergeByNameDto
from src.client.generated.models.custom_field_value_project_patch_dto import CustomFieldValueProjectPatchDto
from src.client.generated.models.custom_field_value_project_rename_dto import CustomFieldValueProjectRenameDto
from src.client.generated.models.custom_field_value_with_cf_dto import CustomFieldValueWithCfDto
from src.client.generated.models.custom_field_value_with_tc_count_dto import CustomFieldValueWithTcCountDto
from src.client.generated.models.custom_field_with_values_dto import CustomFieldWithValuesDto
from src.client.generated.models.dashboard_copy_dto import DashboardCopyDto
from src.client.generated.models.dashboard_create_dto import DashboardCreateDto
from src.client.generated.models.dashboard_drag_and_drop_dto import DashboardDragAndDropDto
from src.client.generated.models.dashboard_dto import DashboardDto
from src.client.generated.models.dashboard_overview_dto import DashboardOverviewDto
from src.client.generated.models.dashboard_patch_dto import DashboardPatchDto
from src.client.generated.models.dashboard_update_dto import DashboardUpdateDto
from src.client.generated.models.default_custom_field_value_dto import DefaultCustomFieldValueDto
from src.client.generated.models.default_text_markup_document import DefaultTextMarkupDocument
from src.client.generated.models.default_text_markup_document_all_of_content import DefaultTextMarkupDocumentAllOfContent
from src.client.generated.models.default_webhook_settings import DefaultWebhookSettings
from src.client.generated.models.default_webhook_settings_all_of_subjects import DefaultWebhookSettingsAllOfSubjects
from src.client.generated.models.defect_bulk_dto import DefectBulkDto
from src.client.generated.models.defect_count_row_dto import DefectCountRowDto
from src.client.generated.models.defect_create_dto import DefectCreateDto
from src.client.generated.models.defect_dto import DefectDto
from src.client.generated.models.defect_issue_link_dto import DefectIssueLinkDto
from src.client.generated.models.defect_matcher_create_dto import DefectMatcherCreateDto
from src.client.generated.models.defect_matcher_dto import DefectMatcherDto
from src.client.generated.models.defect_matcher_patch_dto import DefectMatcherPatchDto
from src.client.generated.models.defect_overview_dto import DefectOverviewDto
from src.client.generated.models.defect_patch_dto import DefectPatchDto
from src.client.generated.models.defect_row_dto import DefectRowDto
from src.client.generated.models.defect_webhook import DefectWebhook
from src.client.generated.models.diff_value_change_boolean import DiffValueChangeBoolean
from src.client.generated.models.diff_value_change_long import DiffValueChangeLong
from src.client.generated.models.diff_value_change_set_long import DiffValueChangeSetLong
from src.client.generated.models.diff_value_change_string import DiffValueChangeString
from src.client.generated.models.document_node import DocumentNode
from src.client.generated.models.env_var_create_dto import EnvVarCreateDto
from src.client.generated.models.env_var_dto import EnvVarDto
from src.client.generated.models.env_var_merge_dto import EnvVarMergeDto
from src.client.generated.models.env_var_patch_dto import EnvVarPatchDto
from src.client.generated.models.env_var_schema_create_dto import EnvVarSchemaCreateDto
from src.client.generated.models.env_var_schema_dto import EnvVarSchemaDto
from src.client.generated.models.env_var_schema_patch_dto import EnvVarSchemaPatchDto
from src.client.generated.models.env_var_value_create_dto import EnvVarValueCreateDto
from src.client.generated.models.env_var_value_dto import EnvVarValueDto
from src.client.generated.models.env_var_value_patch_dto import EnvVarValuePatchDto
from src.client.generated.models.environment_set_dto import EnvironmentSetDto
from src.client.generated.models.event_type import EventType
from src.client.generated.models.event_type_settings import EventTypeSettings
from src.client.generated.models.example_export_field_mapping import ExampleExportFieldMapping
from src.client.generated.models.expected_body_step_dto import ExpectedBodyStepDto
from src.client.generated.models.export_field_mapping_test_case_export_field import ExportFieldMappingTestCaseExportField
from src.client.generated.models.export_field_mapping_test_result_export_field import ExportFieldMappingTestResultExportField
from src.client.generated.models.export_request_dto import ExportRequestDto
from src.client.generated.models.export_status import ExportStatus
from src.client.generated.models.export_type import ExportType
from src.client.generated.models.ext_form_field import ExtFormField
from src.client.generated.models.ext_form_field_checkbox import ExtFormFieldCheckbox
from src.client.generated.models.ext_form_field_composite import ExtFormFieldComposite
from src.client.generated.models.ext_form_field_external_select import ExtFormFieldExternalSelect
from src.client.generated.models.ext_form_field_form_choice import ExtFormFieldFormChoice
from src.client.generated.models.ext_form_field_form_choice_option import ExtFormFieldFormChoiceOption
from src.client.generated.models.ext_form_field_list import ExtFormFieldList
from src.client.generated.models.ext_form_field_number import ExtFormFieldNumber
from src.client.generated.models.ext_form_field_option import ExtFormFieldOption
from src.client.generated.models.ext_form_field_password import ExtFormFieldPassword
from src.client.generated.models.ext_form_field_select import ExtFormFieldSelect
from src.client.generated.models.ext_form_field_slider import ExtFormFieldSlider
from src.client.generated.models.ext_form_field_text import ExtFormFieldText
from src.client.generated.models.ext_form_field_textarea import ExtFormFieldTextarea
from src.client.generated.models.ext_form_field_unknown import ExtFormFieldUnknown
from src.client.generated.models.ext_issue_link import ExtIssueLink
from src.client.generated.models.ext_issue_type import ExtIssueType
from src.client.generated.models.ext_project import ExtProject
from src.client.generated.models.ext_select_value import ExtSelectValue
from src.client.generated.models.external_link_dto import ExternalLinkDto
from src.client.generated.models.external_run_response_dto import ExternalRunResponseDto
from src.client.generated.models.external_run_start_request_dto import ExternalRunStartRequestDto
from src.client.generated.models.external_run_stop_request_dto import ExternalRunStopRequestDto
from src.client.generated.models.file_upload_response_dto import FileUploadResponseDto
from src.client.generated.models.filter_base_set_dto import FilterBaseSetDto
from src.client.generated.models.filter_create_dto import FilterCreateDto
from src.client.generated.models.filter_dto import FilterDto
from src.client.generated.models.filter_patch_dto import FilterPatchDto
from src.client.generated.models.filter_type_dto import FilterTypeDto
from src.client.generated.models.find_all29200_response import FindAll29200Response
from src.client.generated.models.global_permissions_dto import GlobalPermissionsDto
from src.client.generated.models.global_settings_dto import GlobalSettingsDto
from src.client.generated.models.global_settings_project_create_patch_dto import GlobalSettingsProjectCreatePatchDto
from src.client.generated.models.grid_pos_dto import GridPosDto
from src.client.generated.models.group_by import GroupBy
from src.client.generated.models.group_node_context import GroupNodeContext
from src.client.generated.models.id_and_name_only_dto import IdAndNameOnlyDto
from src.client.generated.models.id_only_dto import IdOnlyDto
from src.client.generated.models.import_request_dto import ImportRequestDto
from src.client.generated.models.import_request_info_dto import ImportRequestInfoDto
from src.client.generated.models.import_request_state_dto import ImportRequestStateDto
from src.client.generated.models.import_request_type_dto import ImportRequestTypeDto
from src.client.generated.models.integration_create_dto import IntegrationCreateDto
from src.client.generated.models.integration_dto import IntegrationDto
from src.client.generated.models.integration_export_create_dto import IntegrationExportCreateDto
from src.client.generated.models.integration_export_dto import IntegrationExportDto
from src.client.generated.models.integration_export_patch_dto import IntegrationExportPatchDto
from src.client.generated.models.integration_fields_form_dto import IntegrationFieldsFormDto
from src.client.generated.models.integration_fields_form_dto_default_project_fields_inner import IntegrationFieldsFormDtoDefaultProjectFieldsInner
from src.client.generated.models.integration_info_dto import IntegrationInfoDto
from src.client.generated.models.integration_links_fixed_dto import IntegrationLinksFixedDto
from src.client.generated.models.integration_operation_type_dto import IntegrationOperationTypeDto
from src.client.generated.models.integration_patch_dto import IntegrationPatchDto
from src.client.generated.models.integration_routine_type_dto import IntegrationRoutineTypeDto
from src.client.generated.models.integration_type_dto import IntegrationTypeDto
from src.client.generated.models.integration_validate_dto import IntegrationValidateDto
from src.client.generated.models.integration_webhook_create_dto import IntegrationWebhookCreateDto
from src.client.generated.models.integration_webhook_dto import IntegrationWebhookDto
from src.client.generated.models.integration_webhook_log_dto import IntegrationWebhookLogDto
from src.client.generated.models.integration_webhook_patch_dto import IntegrationWebhookPatchDto
from src.client.generated.models.integration_webhook_token_dto import IntegrationWebhookTokenDto
from src.client.generated.models.issue_create_dto import IssueCreateDto
from src.client.generated.models.issue_dto import IssueDto
from src.client.generated.models.issue_import_field_mapping import IssueImportFieldMapping
from src.client.generated.models.issue_integration_export_field_mapping import IssueIntegrationExportFieldMapping
from src.client.generated.models.issue_patch_dto import IssuePatchDto
from src.client.generated.models.issue_schema_create_dto import IssueSchemaCreateDto
from src.client.generated.models.issue_schema_dto import IssueSchemaDto
from src.client.generated.models.issue_schema_patch_dto import IssueSchemaPatchDto
from src.client.generated.models.issue_to_create_dto import IssueToCreateDto
from src.client.generated.models.italic_mark import ItalicMark
from src.client.generated.models.job import Job
from src.client.generated.models.job_create_dto import JobCreateDto
from src.client.generated.models.job_dto import JobDto
from src.client.generated.models.job_info_dto import JobInfoDto
from src.client.generated.models.job_mapping import JobMapping
from src.client.generated.models.job_parameter_dto import JobParameterDto
from src.client.generated.models.job_patch_dto import JobPatchDto
from src.client.generated.models.job_rerun_request_dto import JobRerunRequestDto
from src.client.generated.models.job_run import JobRun
from src.client.generated.models.job_run_dto import JobRunDto
from src.client.generated.models.job_run_request_dto import JobRunRequestDto
from src.client.generated.models.job_run_stage_dto import JobRunStageDto
from src.client.generated.models.job_run_status_dto import JobRunStatusDto
from src.client.generated.models.job_run_upload_info_dto import JobRunUploadInfoDto
from src.client.generated.models.job_test_cases_stat_dto import JobTestCasesStatDto
from src.client.generated.models.launch import Launch
from src.client.generated.models.launch_cleanup_request import LaunchCleanupRequest
from src.client.generated.models.launch_close_config_dto import LaunchCloseConfigDto
from src.client.generated.models.launch_copy_rq_dto import LaunchCopyRqDto
from src.client.generated.models.launch_create_and_upload_dto import LaunchCreateAndUploadDto
from src.client.generated.models.launch_create_dto import LaunchCreateDto
from src.client.generated.models.launch_diff_cell import LaunchDiffCell
from src.client.generated.models.launch_diff_mode import LaunchDiffMode
from src.client.generated.models.launch_diff_row import LaunchDiffRow
from src.client.generated.models.launch_diff_status_change_dto import LaunchDiffStatusChangeDto
from src.client.generated.models.launch_diff_test_result import LaunchDiffTestResult
from src.client.generated.models.launch_diff_test_result_dto import LaunchDiffTestResultDto
from src.client.generated.models.launch_dto import LaunchDto
from src.client.generated.models.launch_existing_upload_dto import LaunchExistingUploadDto
from src.client.generated.models.launch_live_doc_config_dto import LaunchLiveDocConfigDto
from src.client.generated.models.launch_member_stats_dto import LaunchMemberStatsDto
from src.client.generated.models.launch_merge_dto import LaunchMergeDto
from src.client.generated.models.launch_patch_dto import LaunchPatchDto
from src.client.generated.models.launch_pdf_options import LaunchPdfOptions
from src.client.generated.models.launch_pdf_part import LaunchPdfPart
from src.client.generated.models.launch_pdf_structure import LaunchPdfStructure
from src.client.generated.models.launch_preview_dto import LaunchPreviewDto
from src.client.generated.models.launch_progress_dto import LaunchProgressDto
from src.client.generated.models.launch_row_dto import LaunchRowDto
from src.client.generated.models.launch_tag_create_dto import LaunchTagCreateDto
from src.client.generated.models.launch_tag_dto import LaunchTagDto
from src.client.generated.models.launch_tag_patch_dto import LaunchTagPatchDto
from src.client.generated.models.launch_test_cases_add_dto import LaunchTestCasesAddDto
from src.client.generated.models.launch_test_plan_add_dto import LaunchTestPlanAddDto
from src.client.generated.models.launch_upload_response_dto import LaunchUploadResponseDto
from src.client.generated.models.launch_variable_dto import LaunchVariableDto
from src.client.generated.models.launch_webhook import LaunchWebhook
from src.client.generated.models.line_break_paragraph_node import LineBreakParagraphNode
from src.client.generated.models.link_export_field_mapping import LinkExportFieldMapping
from src.client.generated.models.link_import_field_mapping import LinkImportFieldMapping
from src.client.generated.models.link_mark import LinkMark
from src.client.generated.models.list_selection_dto import ListSelectionDto
from src.client.generated.models.manual_session_request_dto import ManualSessionRequestDto
from src.client.generated.models.matcher import Matcher
from src.client.generated.models.member_create_dto import MemberCreateDto
from src.client.generated.models.member_dto import MemberDto
from src.client.generated.models.member_patch_dto import MemberPatchDto
from src.client.generated.models.metric import Metric
from src.client.generated.models.mfa_global_settings_dto import MfaGlobalSettingsDto
from src.client.generated.models.mfa_global_settings_patch_dto import MfaGlobalSettingsPatchDto
from src.client.generated.models.multi_value_export_field_mapping import MultiValueExportFieldMapping
from src.client.generated.models.multi_value_import_field_mapping import MultiValueImportFieldMapping
from src.client.generated.models.multi_value_map_string_string import MultiValueMapStringString
from src.client.generated.models.mute_create_dto import MuteCreateDto
from src.client.generated.models.mute_dto import MuteDto
from src.client.generated.models.name_only_dto import NameOnlyDto
from src.client.generated.models.node_type import NodeType
from src.client.generated.models.normalized_scenario_dto import NormalizedScenarioDto
from src.client.generated.models.normalized_scenario_dto_attachments_value import NormalizedScenarioDtoAttachmentsValue
from src.client.generated.models.normalized_scenario_step_dto import NormalizedScenarioStepDto
from src.client.generated.models.ordered_value_dto_string import OrderedValueDtoString
from src.client.generated.models.page_access_group_dto import PageAccessGroupDto
from src.client.generated.models.page_access_group_dto_sort import PageAccessGroupDtoSort
from src.client.generated.models.page_access_group_pa_dto import PageAccessGroupPaDto
from src.client.generated.models.page_access_group_user_dto import PageAccessGroupUserDto
from src.client.generated.models.page_async_job_dto import PageAsyncJobDto
from src.client.generated.models.page_category_dto import PageCategoryDto
from src.client.generated.models.page_category_matcher_dto import PageCategoryMatcherDto
from src.client.generated.models.page_cleaner_schema_dto import PageCleanerSchemaDto
from src.client.generated.models.page_comment_dto import PageCommentDto
from src.client.generated.models.page_custom_field_project_dto import PageCustomFieldProjectDto
from src.client.generated.models.page_custom_field_schema_dto import PageCustomFieldSchemaDto
from src.client.generated.models.page_custom_field_value_with_tc_count_dto import PageCustomFieldValueWithTcCountDto
from src.client.generated.models.page_dashboard_dto import PageDashboardDto
from src.client.generated.models.page_defect_count_row_dto import PageDefectCountRowDto
from src.client.generated.models.page_defect_dto import PageDefectDto
from src.client.generated.models.page_defect_matcher_dto import PageDefectMatcherDto
from src.client.generated.models.page_defect_row_dto import PageDefectRowDto
from src.client.generated.models.page_env_var_schema_dto import PageEnvVarSchemaDto
from src.client.generated.models.page_export_request_dto import PageExportRequestDto
from src.client.generated.models.page_filter_dto import PageFilterDto
from src.client.generated.models.page_id_and_name_only_dto import PageIdAndNameOnlyDto
from src.client.generated.models.page_import_request_dto import PageImportRequestDto
from src.client.generated.models.page_integration_dto import PageIntegrationDto
from src.client.generated.models.page_integration_export_dto import PageIntegrationExportDto
from src.client.generated.models.page_integration_info_dto import PageIntegrationInfoDto
from src.client.generated.models.page_integration_webhook_dto import PageIntegrationWebhookDto
from src.client.generated.models.page_integration_webhook_log_dto import PageIntegrationWebhookLogDto
from src.client.generated.models.page_issue_dto import PageIssueDto
from src.client.generated.models.page_issue_schema_dto import PageIssueSchemaDto
from src.client.generated.models.page_job_dto import PageJobDto
from src.client.generated.models.page_launch_diff_row import PageLaunchDiffRow
from src.client.generated.models.page_launch_dto import PageLaunchDto
from src.client.generated.models.page_launch_member_stats_dto import PageLaunchMemberStatsDto
from src.client.generated.models.page_launch_preview_dto import PageLaunchPreviewDto
from src.client.generated.models.page_launch_row_dto import PageLaunchRowDto
from src.client.generated.models.page_launch_tag_dto import PageLaunchTagDto
from src.client.generated.models.page_launch_variable_dto import PageLaunchVariableDto
from src.client.generated.models.page_member_dto import PageMemberDto
from src.client.generated.models.page_mute_dto import PageMuteDto
from src.client.generated.models.page_permission_set_dto import PagePermissionSetDto
from src.client.generated.models.page_project_collaborator_access_dto import PageProjectCollaboratorAccessDto
from src.client.generated.models.page_project_collaborator_dto import PageProjectCollaboratorDto
from src.client.generated.models.page_project_custom_field_dto import PageProjectCustomFieldDto
from src.client.generated.models.page_project_dto import PageProjectDto
from src.client.generated.models.page_project_group_access_dto import PageProjectGroupAccessDto
from src.client.generated.models.page_project_integration_dto import PageProjectIntegrationDto
from src.client.generated.models.page_project_suggest_dto import PageProjectSuggestDto
from src.client.generated.models.page_role_schema_dto import PageRoleSchemaDto
from src.client.generated.models.page_shared_step_attachment_row_dto import PageSharedStepAttachmentRowDto
from src.client.generated.models.page_shared_step_dto import PageSharedStepDto
from src.client.generated.models.page_statistic_widget_item import PageStatisticWidgetItem
from src.client.generated.models.page_status_dto import PageStatusDto
from src.client.generated.models.page_template_dto import PageTemplateDto
from src.client.generated.models.page_test_case_attachment_row_dto import PageTestCaseAttachmentRowDto
from src.client.generated.models.page_test_case_audit_log_entry_dto import PageTestCaseAuditLogEntryDto
from src.client.generated.models.page_test_case_dto import PageTestCaseDto
from src.client.generated.models.page_test_case_example_dto import PageTestCaseExampleDto
from src.client.generated.models.page_test_case_flat_dto import PageTestCaseFlatDto
from src.client.generated.models.page_test_case_row_dto import PageTestCaseRowDto
from src.client.generated.models.page_test_case_tree_group_dto import PageTestCaseTreeGroupDto
from src.client.generated.models.page_test_case_tree_leaf_dto import PageTestCaseTreeLeafDto
from src.client.generated.models.page_test_case_tree_node_dto import PageTestCaseTreeNodeDto
from src.client.generated.models.page_test_case_tree_node_dto_content_inner import PageTestCaseTreeNodeDtoContentInner
from src.client.generated.models.page_test_fixture_result_attachment_row_dto import PageTestFixtureResultAttachmentRowDto
from src.client.generated.models.page_test_key_schema_dto import PageTestKeySchemaDto
from src.client.generated.models.page_test_layer_dto import PageTestLayerDto
from src.client.generated.models.page_test_layer_schema_dto import PageTestLayerSchemaDto
from src.client.generated.models.page_test_plan_dto import PageTestPlanDto
from src.client.generated.models.page_test_plan_row_dto import PageTestPlanRowDto
from src.client.generated.models.page_test_result_attachment_row_dto import PageTestResultAttachmentRowDto
from src.client.generated.models.page_test_result_defect_match_dto import PageTestResultDefectMatchDto
from src.client.generated.models.page_test_result_dto import PageTestResultDto
from src.client.generated.models.page_test_result_flat_dto import PageTestResultFlatDto
from src.client.generated.models.page_test_result_history_dto import PageTestResultHistoryDto
from src.client.generated.models.page_test_result_retries_row_dto import PageTestResultRetriesRowDto
from src.client.generated.models.page_test_result_row_dto import PageTestResultRowDto
from src.client.generated.models.page_test_result_tree_group_dto import PageTestResultTreeGroupDto
from src.client.generated.models.page_test_result_tree_leaf_dto import PageTestResultTreeLeafDto
from src.client.generated.models.page_test_result_tree_node_dto import PageTestResultTreeNodeDto
from src.client.generated.models.page_test_result_tree_node_dto_content_inner import PageTestResultTreeNodeDtoContentInner
from src.client.generated.models.page_tree_dto import PageTreeDto
from src.client.generated.models.page_tree_dto_v2 import PageTreeDtoV2
from src.client.generated.models.page_tree_path_dto import PageTreePathDto
from src.client.generated.models.page_webhook_dto import PageWebhookDto
from src.client.generated.models.page_widget_dto import PageWidgetDto
from src.client.generated.models.page_workflow_dto import PageWorkflowDto
from src.client.generated.models.page_workflow_schema_dto import PageWorkflowSchemaDto
from src.client.generated.models.pageable import Pageable
from src.client.generated.models.paragraph_document_node import ParagraphDocumentNode
from src.client.generated.models.paragraph_document_node_all_of_content import ParagraphDocumentNodeAllOfContent
from src.client.generated.models.paragraph_node import ParagraphNode
from src.client.generated.models.parameter import Parameter
from src.client.generated.models.parameter_value_dto import ParameterValueDto
from src.client.generated.models.permission_dto import PermissionDto
from src.client.generated.models.permission_set_create_dto import PermissionSetCreateDto
from src.client.generated.models.permission_set_dto import PermissionSetDto
from src.client.generated.models.permission_set_patch_dto import PermissionSetPatchDto
from src.client.generated.models.project_access_collaborator_add_dto import ProjectAccessCollaboratorAddDto
from src.client.generated.models.project_access_group_add_dto import ProjectAccessGroupAddDto
from src.client.generated.models.project_category_add_dto import ProjectCategoryAddDto
from src.client.generated.models.project_category_matcher_add_dto import ProjectCategoryMatcherAddDto
from src.client.generated.models.project_category_matcher_remove_dto import ProjectCategoryMatcherRemoveDto
from src.client.generated.models.project_category_remove_dto import ProjectCategoryRemoveDto
from src.client.generated.models.project_collaborator_access_dto import ProjectCollaboratorAccessDto
from src.client.generated.models.project_collaborator_dto import ProjectCollaboratorDto
from src.client.generated.models.project_create_dto import ProjectCreateDto
from src.client.generated.models.project_custom_field_dto import ProjectCustomFieldDto
from src.client.generated.models.project_dto import ProjectDto
from src.client.generated.models.project_group_access_add_dto import ProjectGroupAccessAddDto
from src.client.generated.models.project_group_access_dto import ProjectGroupAccessDto
from src.client.generated.models.project_integration_create_dto import ProjectIntegrationCreateDto
from src.client.generated.models.project_integration_dto import ProjectIntegrationDto
from src.client.generated.models.project_integration_fields_form_dto import ProjectIntegrationFieldsFormDto
from src.client.generated.models.project_integration_patch_dto import ProjectIntegrationPatchDto
from src.client.generated.models.project_integration_validate_dto import ProjectIntegrationValidateDto
from src.client.generated.models.project_metric_dto import ProjectMetricDto
from src.client.generated.models.project_patch_dto import ProjectPatchDto
from src.client.generated.models.project_property_create_dto import ProjectPropertyCreateDto
from src.client.generated.models.project_property_dto import ProjectPropertyDto
from src.client.generated.models.project_property_patch_dto import ProjectPropertyPatchDto
from src.client.generated.models.project_stats_dto import ProjectStatsDto
from src.client.generated.models.project_suggest_dto import ProjectSuggestDto
from src.client.generated.models.project_sync_request import ProjectSyncRequest
from src.client.generated.models.project_test_case_count_dto import ProjectTestCaseCountDto
from src.client.generated.models.resolve_request_v2_dto import ResolveRequestV2Dto
from src.client.generated.models.retry import Retry
from src.client.generated.models.role_create_dto import RoleCreateDto
from src.client.generated.models.role_dto import RoleDto
from src.client.generated.models.role_export_field_mapping import RoleExportFieldMapping
from src.client.generated.models.role_import_field_mapping import RoleImportFieldMapping
from src.client.generated.models.role_patch_dto import RolePatchDto
from src.client.generated.models.role_schema_create_dto import RoleSchemaCreateDto
from src.client.generated.models.role_schema_dto import RoleSchemaDto
from src.client.generated.models.role_schema_patch_dto import RoleSchemaPatchDto
from src.client.generated.models.scenario_export_field_mapping import ScenarioExportFieldMapping
from src.client.generated.models.scenario_step_copy_dto import ScenarioStepCopyDto
from src.client.generated.models.scenario_step_create_dto import ScenarioStepCreateDto
from src.client.generated.models.scenario_step_created_response_dto import ScenarioStepCreatedResponseDto
from src.client.generated.models.scenario_step_dto import ScenarioStepDto
from src.client.generated.models.scenario_step_move_dto import ScenarioStepMoveDto
from src.client.generated.models.scenario_step_patch_dto import ScenarioStepPatchDto
from src.client.generated.models.session_variable import SessionVariable
from src.client.generated.models.shared_step_attachment_patch_dto import SharedStepAttachmentPatchDto
from src.client.generated.models.shared_step_attachment_row_dto import SharedStepAttachmentRowDto
from src.client.generated.models.shared_step_create_dto import SharedStepCreateDto
from src.client.generated.models.shared_step_dto import SharedStepDto
from src.client.generated.models.shared_step_patch_dto import SharedStepPatchDto
from src.client.generated.models.shared_step_scenario_dto import SharedStepScenarioDto
from src.client.generated.models.shared_step_scenario_dto_steps_inner import SharedStepScenarioDtoStepsInner
from src.client.generated.models.shared_step_step_dto import SharedStepStepDto
from src.client.generated.models.statistic_dto import StatisticDto
from src.client.generated.models.statistic_widget_item import StatisticWidgetItem
from src.client.generated.models.status import Status
from src.client.generated.models.status_create_dto import StatusCreateDto
from src.client.generated.models.status_dto import StatusDto
from src.client.generated.models.status_patch_dto import StatusPatchDto
from src.client.generated.models.strike_mark import StrikeMark
from src.client.generated.models.tag_import_field_mapping import TagImportFieldMapping
from src.client.generated.models.template_dto import TemplateDto
from src.client.generated.models.template_overview_dto import TemplateOverviewDto
from src.client.generated.models.test_case_association_diff import TestCaseAssociationDiff
from src.client.generated.models.test_case_attachment_patch_dto import TestCaseAttachmentPatchDto
from src.client.generated.models.test_case_attachment_row_dto import TestCaseAttachmentRowDto
from src.client.generated.models.test_case_audit_log_data import TestCaseAuditLogData
from src.client.generated.models.test_case_audit_log_data_diff import TestCaseAuditLogDataDiff
from src.client.generated.models.test_case_audit_log_entry_dto import TestCaseAuditLogEntryDto
from src.client.generated.models.test_case_bulk_cf_move_strategy import TestCaseBulkCfMoveStrategy
from src.client.generated.models.test_case_bulk_change_project_dto_v2 import TestCaseBulkChangeProjectDtoV2
from src.client.generated.models.test_case_bulk_clone_dto import TestCaseBulkCloneDto
from src.client.generated.models.test_case_bulk_clone_dto_v2 import TestCaseBulkCloneDtoV2
from src.client.generated.models.test_case_bulk_drag_and_drop_dto import TestCaseBulkDragAndDropDto
from src.client.generated.models.test_case_bulk_drag_and_drop_dto_v2 import TestCaseBulkDragAndDropDtoV2
from src.client.generated.models.test_case_bulk_dto import TestCaseBulkDto
from src.client.generated.models.test_case_bulk_dto_v2 import TestCaseBulkDtoV2
from src.client.generated.models.test_case_bulk_entity_ids_dto import TestCaseBulkEntityIdsDto
from src.client.generated.models.test_case_bulk_entity_ids_dto_v2 import TestCaseBulkEntityIdsDtoV2
from src.client.generated.models.test_case_bulk_export_csv_options import TestCaseBulkExportCsvOptions
from src.client.generated.models.test_case_bulk_export_csv_options_mapping_inner import TestCaseBulkExportCsvOptionsMappingInner
from src.client.generated.models.test_case_bulk_export_pdf_options import TestCaseBulkExportPdfOptions
from src.client.generated.models.test_case_bulk_external_link_dto import TestCaseBulkExternalLinkDto
from src.client.generated.models.test_case_bulk_issue_dto import TestCaseBulkIssueDto
from src.client.generated.models.test_case_bulk_layer_dto import TestCaseBulkLayerDto
from src.client.generated.models.test_case_bulk_member_dto import TestCaseBulkMemberDto
from src.client.generated.models.test_case_bulk_mute_dto import TestCaseBulkMuteDto
from src.client.generated.models.test_case_bulk_new_cfv_dto import TestCaseBulkNewCfvDto
from src.client.generated.models.test_case_bulk_project_change_dto import TestCaseBulkProjectChangeDto
from src.client.generated.models.test_case_bulk_run_existing_launch_dto import TestCaseBulkRunExistingLaunchDto
from src.client.generated.models.test_case_bulk_run_new_launch_dto import TestCaseBulkRunNewLaunchDto
from src.client.generated.models.test_case_bulk_status_dto import TestCaseBulkStatusDto
from src.client.generated.models.test_case_bulk_tag_dto import TestCaseBulkTagDto
from src.client.generated.models.test_case_bulk_test_plan_create_dto import TestCaseBulkTestPlanCreateDto
from src.client.generated.models.test_case_cf_bulk_delta_dto_v2 import TestCaseCfBulkDeltaDtoV2
from src.client.generated.models.test_case_cfv_bulk_add_dto import TestCaseCfvBulkAddDto
from src.client.generated.models.test_case_clone_rq_dto import TestCaseCloneRqDto
from src.client.generated.models.test_case_count_dto import TestCaseCountDto
from src.client.generated.models.test_case_create_v2_dto import TestCaseCreateV2Dto
from src.client.generated.models.test_case_csv_export_options import TestCaseCsvExportOptions
from src.client.generated.models.test_case_csv_import_options import TestCaseCsvImportOptions
from src.client.generated.models.test_case_csv_import_options_mapping_inner import TestCaseCsvImportOptionsMappingInner
from src.client.generated.models.test_case_csv_preview_options import TestCaseCsvPreviewOptions
from src.client.generated.models.test_case_detach_automation_rq_dto import TestCaseDetachAutomationRqDto
from src.client.generated.models.test_case_diff import TestCaseDiff
from src.client.generated.models.test_case_dto import TestCaseDto
from src.client.generated.models.test_case_example_dto import TestCaseExampleDto
from src.client.generated.models.test_case_export_field import TestCaseExportField
from src.client.generated.models.test_case_external_link_bulk_add_dto import TestCaseExternalLinkBulkAddDto
from src.client.generated.models.test_case_flat_dto import TestCaseFlatDto
from src.client.generated.models.test_case_full_tree_node_dto import TestCaseFullTreeNodeDto
from src.client.generated.models.test_case_import_dto import TestCaseImportDto
from src.client.generated.models.test_case_import_field import TestCaseImportField
from src.client.generated.models.test_case_import_mapping import TestCaseImportMapping
from src.client.generated.models.test_case_info import TestCaseInfo
from src.client.generated.models.test_case_issue_bulk_add_dto import TestCaseIssueBulkAddDto
from src.client.generated.models.test_case_last_result_dto import TestCaseLastResultDto
from src.client.generated.models.test_case_layer_bulk_set_dto import TestCaseLayerBulkSetDto
from src.client.generated.models.test_case_light_tree_node_dto import TestCaseLightTreeNodeDto
from src.client.generated.models.test_case_member_bulk_add_dto import TestCaseMemberBulkAddDto
from src.client.generated.models.test_case_mute_bulk_add_dto import TestCaseMuteBulkAddDto
from src.client.generated.models.test_case_overview_dto import TestCaseOverviewDto
from src.client.generated.models.test_case_parameter_dto import TestCaseParameterDto
from src.client.generated.models.test_case_parameter_values import TestCaseParameterValues
from src.client.generated.models.test_case_patch_v2_dto import TestCasePatchV2Dto
from src.client.generated.models.test_case_pdf_options import TestCasePdfOptions
from src.client.generated.models.test_case_pdf_part import TestCasePdfPart
from src.client.generated.models.test_case_relation_dto import TestCaseRelationDto
from src.client.generated.models.test_case_relation_type_dto import TestCaseRelationTypeDto
from src.client.generated.models.test_case_row_dto import TestCaseRowDto
from src.client.generated.models.test_case_run_bulk_existing_launch_dto import TestCaseRunBulkExistingLaunchDto
from src.client.generated.models.test_case_run_bulk_new_launch_dto import TestCaseRunBulkNewLaunchDto
from src.client.generated.models.test_case_run_by_stats import TestCaseRunByStats
from src.client.generated.models.test_case_run_stats_request_dto import TestCaseRunStatsRequestDto
from src.client.generated.models.test_case_scenario_dto import TestCaseScenarioDto
from src.client.generated.models.test_case_scenario_step_dto import TestCaseScenarioStepDto
from src.client.generated.models.test_case_scenario_v2_dto import TestCaseScenarioV2Dto
from src.client.generated.models.test_case_selection_dto_v2 import TestCaseSelectionDtoV2
from src.client.generated.models.test_case_status_bulk_set_dto import TestCaseStatusBulkSetDto
from src.client.generated.models.test_case_style import TestCaseStyle
from src.client.generated.models.test_case_sync_from_to import TestCaseSyncFromTo
from src.client.generated.models.test_case_sync_rq_dto import TestCaseSyncRqDto
from src.client.generated.models.test_case_tag_bulk_add_dto import TestCaseTagBulkAddDto
from src.client.generated.models.test_case_test_plan_bulk_create_dto import TestCaseTestPlanBulkCreateDto
from src.client.generated.models.test_case_tree_all_paths_dto import TestCaseTreeAllPathsDto
from src.client.generated.models.test_case_tree_filter_count_dto import TestCaseTreeFilterCountDto
from src.client.generated.models.test_case_tree_group_add_dto import TestCaseTreeGroupAddDto
from src.client.generated.models.test_case_tree_group_dto import TestCaseTreeGroupDto
from src.client.generated.models.test_case_tree_group_rename_dto import TestCaseTreeGroupRenameDto
from src.client.generated.models.test_case_tree_leaf_add_dto import TestCaseTreeLeafAddDto
from src.client.generated.models.test_case_tree_leaf_dto import TestCaseTreeLeafDto
from src.client.generated.models.test_case_tree_leaf_dto_v2 import TestCaseTreeLeafDtoV2
from src.client.generated.models.test_case_tree_leaf_rename_dto import TestCaseTreeLeafRenameDto
from src.client.generated.models.test_case_tree_node_dto import TestCaseTreeNodeDto
from src.client.generated.models.test_case_tree_run_stat_request_dto import TestCaseTreeRunStatRequestDto
from src.client.generated.models.test_case_tree_selection_dto import TestCaseTreeSelectionDto
from src.client.generated.models.test_case_tree_selection_dto_v2 import TestCaseTreeSelectionDtoV2
from src.client.generated.models.test_case_type import TestCaseType
from src.client.generated.models.test_case_update_field_dto import TestCaseUpdateFieldDto
from src.client.generated.models.test_case_update_policy_dto import TestCaseUpdatePolicyDto
from src.client.generated.models.test_case_update_schema_create_dto import TestCaseUpdateSchemaCreateDto
from src.client.generated.models.test_case_update_schema_dto import TestCaseUpdateSchemaDto
from src.client.generated.models.test_case_update_schema_patch_dto import TestCaseUpdateSchemaPatchDto
from src.client.generated.models.test_case_version_dto import TestCaseVersionDto
from src.client.generated.models.test_case_version_patch_dto import TestCaseVersionPatchDto
from src.client.generated.models.test_case_version_rq_dto import TestCaseVersionRqDto
from src.client.generated.models.test_duration_count import TestDurationCount
from src.client.generated.models.test_fixture_result_attachment_patch_dto import TestFixtureResultAttachmentPatchDto
from src.client.generated.models.test_fixture_result_attachment_row_dto import TestFixtureResultAttachmentRowDto
from src.client.generated.models.test_fixture_result_type_dto import TestFixtureResultTypeDto
from src.client.generated.models.test_fixture_result_v2_dto import TestFixtureResultV2Dto
from src.client.generated.models.test_key_dto import TestKeyDto
from src.client.generated.models.test_key_import_field_mapping import TestKeyImportFieldMapping
from src.client.generated.models.test_key_schema_create_dto import TestKeySchemaCreateDto
from src.client.generated.models.test_key_schema_dto import TestKeySchemaDto
from src.client.generated.models.test_key_schema_patch_dto import TestKeySchemaPatchDto
from src.client.generated.models.test_layer_create_dto import TestLayerCreateDto
from src.client.generated.models.test_layer_dto import TestLayerDto
from src.client.generated.models.test_layer_patch_dto import TestLayerPatchDto
from src.client.generated.models.test_layer_schema_create_dto import TestLayerSchemaCreateDto
from src.client.generated.models.test_layer_schema_dto import TestLayerSchemaDto
from src.client.generated.models.test_layer_schema_patch_dto import TestLayerSchemaPatchDto
from src.client.generated.models.test_plan_assign_dto import TestPlanAssignDto
from src.client.generated.models.test_plan_create_dto import TestPlanCreateDto
from src.client.generated.models.test_plan_diff_dto import TestPlanDiffDto
from src.client.generated.models.test_plan_dto import TestPlanDto
from src.client.generated.models.test_plan_job_parameters_dto import TestPlanJobParametersDto
from src.client.generated.models.test_plan_job_stat_dto import TestPlanJobStatDto
from src.client.generated.models.test_plan_member_stat_dto import TestPlanMemberStatDto
from src.client.generated.models.test_plan_patch_dto import TestPlanPatchDto
from src.client.generated.models.test_plan_row_dto import TestPlanRowDto
from src.client.generated.models.test_plan_run_request_dto import TestPlanRunRequestDto
from src.client.generated.models.test_plan_tree_selection_dto import TestPlanTreeSelectionDto
from src.client.generated.models.test_plan_webhook import TestPlanWebhook
from src.client.generated.models.test_result_attachment_patch_dto import TestResultAttachmentPatchDto
from src.client.generated.models.test_result_attachment_row_dto import TestResultAttachmentRowDto
from src.client.generated.models.test_result_attachment_step_dto import TestResultAttachmentStepDto
from src.client.generated.models.test_result_attachment_step_dto_all_of_attachment import TestResultAttachmentStepDtoAllOfAttachment
from src.client.generated.models.test_result_body_step_dto import TestResultBodyStepDto
from src.client.generated.models.test_result_bulk_assign_dto import TestResultBulkAssignDto
from src.client.generated.models.test_result_bulk_dto import TestResultBulkDto
from src.client.generated.models.test_result_bulk_entity_ids_dto import TestResultBulkEntityIdsDto
from src.client.generated.models.test_result_bulk_mute_dto import TestResultBulkMuteDto
from src.client.generated.models.test_result_bulk_rerun_dto import TestResultBulkRerunDto
from src.client.generated.models.test_result_bulk_resolve_dto import TestResultBulkResolveDto
from src.client.generated.models.test_result_bulk_tag_dto import TestResultBulkTagDto
from src.client.generated.models.test_result_create_v2_dto import TestResultCreateV2Dto
from src.client.generated.models.test_result_csv_export_options import TestResultCsvExportOptions
from src.client.generated.models.test_result_csv_export_options_mapping_inner import TestResultCsvExportOptionsMappingInner
from src.client.generated.models.test_result_defect_match_dto import TestResultDefectMatchDto
from src.client.generated.models.test_result_defect_matcher_dto import TestResultDefectMatcherDto
from src.client.generated.models.test_result_dto import TestResultDto
from src.client.generated.models.test_result_expected_body_step_dto import TestResultExpectedBodyStepDto
from src.client.generated.models.test_result_export_field import TestResultExportField
from src.client.generated.models.test_result_flat_dto import TestResultFlatDto
from src.client.generated.models.test_result_group_node import TestResultGroupNode
from src.client.generated.models.test_result_history_dto import TestResultHistoryDto
from src.client.generated.models.test_result_leaf_node import TestResultLeafNode
from src.client.generated.models.test_result_link_defect_dto import TestResultLinkDefectDto
from src.client.generated.models.test_result_mute_reason import TestResultMuteReason
from src.client.generated.models.test_result_new_defect_dto import TestResultNewDefectDto
from src.client.generated.models.test_result_parameter_dto import TestResultParameterDto
from src.client.generated.models.test_result_patch_dto import TestResultPatchDto
from src.client.generated.models.test_result_rerun_dto import TestResultRerunDto
from src.client.generated.models.test_result_retries_row_dto import TestResultRetriesRowDto
from src.client.generated.models.test_result_row_dto import TestResultRowDto
from src.client.generated.models.test_result_scenario_dto import TestResultScenarioDto
from src.client.generated.models.test_result_scenario_step_dto import TestResultScenarioStepDto
from src.client.generated.models.test_result_scenario_step_parameter import TestResultScenarioStepParameter
from src.client.generated.models.test_result_scenario_v2_dto import TestResultScenarioV2Dto
from src.client.generated.models.test_result_scenario_v2_dto_steps_inner import TestResultScenarioV2DtoStepsInner
from src.client.generated.models.test_result_stats_dto import TestResultStatsDto
from src.client.generated.models.test_result_tree import TestResultTree
from src.client.generated.models.test_result_tree_filter_count_dto import TestResultTreeFilterCountDto
from src.client.generated.models.test_result_tree_group_dto import TestResultTreeGroupDto
from src.client.generated.models.test_result_tree_group_dto_v2 import TestResultTreeGroupDtoV2
from src.client.generated.models.test_result_tree_leaf_dto import TestResultTreeLeafDto
from src.client.generated.models.test_result_tree_leaf_dto_v2 import TestResultTreeLeafDtoV2
from src.client.generated.models.test_result_tree_node_dto import TestResultTreeNodeDto
from src.client.generated.models.test_result_tree_selection_dto import TestResultTreeSelectionDto
from src.client.generated.models.test_session_response_dto import TestSessionResponseDto
from src.client.generated.models.test_status import TestStatus
from src.client.generated.models.test_status_count import TestStatusCount
from src.client.generated.models.test_tag_create_dto import TestTagCreateDto
from src.client.generated.models.test_tag_dto import TestTagDto
from src.client.generated.models.test_tag_patch_dto import TestTagPatchDto
from src.client.generated.models.text_color_mark import TextColorMark
from src.client.generated.models.text_fill_mark import TextFillMark
from src.client.generated.models.text_mark import TextMark
from src.client.generated.models.text_markup_document import TextMarkupDocument
from src.client.generated.models.text_paragraph_node import TextParagraphNode
from src.client.generated.models.text_paragraph_node_marks_inner import TextParagraphNodeMarksInner
from src.client.generated.models.tr_scenario_step_dto import TrScenarioStepDto
from src.client.generated.models.tree_create_dto import TreeCreateDto
from src.client.generated.models.tree_custom_field_project_dto import TreeCustomFieldProjectDto
from src.client.generated.models.tree_dto import TreeDto
from src.client.generated.models.tree_dto_v2 import TreeDtoV2
from src.client.generated.models.tree_patch_dto import TreePatchDto
from src.client.generated.models.tree_path_create_dto import TreePathCreateDto
from src.client.generated.models.tree_path_dto import TreePathDto
from src.client.generated.models.tree_path_item_dto import TreePathItemDto
from src.client.generated.models.tree_selection_dto import TreeSelectionDto
from src.client.generated.models.underline_mark import UnderlineMark
from src.client.generated.models.upload_attachment_dto import UploadAttachmentDto
from src.client.generated.models.upload_fixtures_results_dto import UploadFixturesResultsDto
from src.client.generated.models.upload_label_dto import UploadLabelDto
from src.client.generated.models.upload_link_dto import UploadLinkDto
from src.client.generated.models.upload_parameter_dto import UploadParameterDto
from src.client.generated.models.upload_results_dto import UploadResultsDto
from src.client.generated.models.upload_results_response_dto import UploadResultsResponseDto
from src.client.generated.models.upload_test_case_dto import UploadTestCaseDto
from src.client.generated.models.upload_test_fixture_result_dto import UploadTestFixtureResultDto
from src.client.generated.models.upload_test_fixture_result_dto_steps_inner import UploadTestFixtureResultDtoStepsInner
from src.client.generated.models.upload_test_fixture_type import UploadTestFixtureType
from src.client.generated.models.upload_test_result_attachment_step_dto import UploadTestResultAttachmentStepDto
from src.client.generated.models.upload_test_result_body_step_dto import UploadTestResultBodyStepDto
from src.client.generated.models.upload_test_result_dto import UploadTestResultDto
from src.client.generated.models.upload_test_result_expected_body_step_dto import UploadTestResultExpectedBodyStepDto
from src.client.generated.models.upload_test_result_step_dto import UploadTestResultStepDto
from src.client.generated.models.upload_test_status import UploadTestStatus
from src.client.generated.models.webhook_create_dto import WebhookCreateDto
from src.client.generated.models.webhook_dto import WebhookDto
from src.client.generated.models.webhook_patch_dto import WebhookPatchDto
from src.client.generated.models.webhook_settings import WebhookSettings
from src.client.generated.models.webhook_subject import WebhookSubject
from src.client.generated.models.webhook_validate_dto import WebhookValidateDto
from src.client.generated.models.widget_create_dto import WidgetCreateDto
from src.client.generated.models.widget_create_dto_options import WidgetCreateDtoOptions
from src.client.generated.models.widget_data_dto import WidgetDataDto
from src.client.generated.models.widget_dto import WidgetDto
from src.client.generated.models.widget_options import WidgetOptions
from src.client.generated.models.widget_options_analytic_graph import WidgetOptionsAnalyticGraph
from src.client.generated.models.widget_options_having_settable_aql import WidgetOptionsHavingSettableAql
from src.client.generated.models.widget_options_last_launch_pie_chart import WidgetOptionsLastLaunchPieChart
from src.client.generated.models.widget_options_launch_list import WidgetOptionsLaunchList
from src.client.generated.models.widget_options_launch_trend import WidgetOptionsLaunchTrend
from src.client.generated.models.widget_options_markdown import WidgetOptionsMarkdown
from src.client.generated.models.widget_options_statistic_trend import WidgetOptionsStatisticTrend
from src.client.generated.models.widget_options_tc_pie_chart import WidgetOptionsTcPieChart
from src.client.generated.models.widget_options_tc_tree_map import WidgetOptionsTcTreeMap
from src.client.generated.models.widget_options_top_tc import WidgetOptionsTopTc
from src.client.generated.models.widget_options_trend import WidgetOptionsTrend
from src.client.generated.models.widget_options_trend_v2 import WidgetOptionsTrendV2
from src.client.generated.models.widget_patch_dto import WidgetPatchDto
from src.client.generated.models.widget_type_dto import WidgetTypeDto
from src.client.generated.models.workflow_create_dto import WorkflowCreateDto
from src.client.generated.models.workflow_dto import WorkflowDto
from src.client.generated.models.workflow_patch_dto import WorkflowPatchDto
from src.client.generated.models.workflow_row_dto import WorkflowRowDto
from src.client.generated.models.workflow_schema_create_dto import WorkflowSchemaCreateDto
from src.client.generated.models.workflow_schema_dto import WorkflowSchemaDto
from src.client.generated.models.workflow_schema_patch_dto import WorkflowSchemaPatchDto




# flake8: noqa

# import apis into api package
from src.client.generated.api.custom_field_controller_api import CustomFieldControllerApi
from src.client.generated.api.custom_field_project_controller_api import CustomFieldProjectControllerApi
from src.client.generated.api.custom_field_project_controller_v2_api import CustomFieldProjectControllerV2Api
from src.client.generated.api.custom_field_schema_controller_api import CustomFieldSchemaControllerApi
from src.client.generated.api.custom_field_value_controller_api import CustomFieldValueControllerApi
from src.client.generated.api.custom_field_value_project_controller_api import CustomFieldValueProjectControllerApi
from src.client.generated.api.defect_controller_api import DefectControllerApi
from src.client.generated.api.defect_matcher_controller_api import DefectMatcherControllerApi
from src.client.generated.api.integration_controller_api import IntegrationControllerApi
from src.client.generated.api.launch_controller_api import LaunchControllerApi
from src.client.generated.api.launch_search_controller_api import LaunchSearchControllerApi
from src.client.generated.api.project_controller_api import ProjectControllerApi
from src.client.generated.api.shared_step_attachment_controller_api import SharedStepAttachmentControllerApi
from src.client.generated.api.shared_step_controller_api import SharedStepControllerApi
from src.client.generated.api.shared_step_scenario_controller_api import SharedStepScenarioControllerApi
from src.client.generated.api.status_controller_api import StatusControllerApi
from src.client.generated.api.test_case_attachment_controller_api import TestCaseAttachmentControllerApi
from src.client.generated.api.test_case_bulk_controller_api import TestCaseBulkControllerApi
from src.client.generated.api.test_case_controller_api import TestCaseControllerApi
from src.client.generated.api.test_case_custom_field_controller_api import TestCaseCustomFieldControllerApi
from src.client.generated.api.test_case_overview_controller_api import TestCaseOverviewControllerApi
from src.client.generated.api.test_case_scenario_controller_api import TestCaseScenarioControllerApi
from src.client.generated.api.test_case_search_controller_api import TestCaseSearchControllerApi
from src.client.generated.api.test_case_tag_controller_api import TestCaseTagControllerApi
from src.client.generated.api.test_case_test_plan_bulk_controller_api import TestCaseTestPlanBulkControllerApi
from src.client.generated.api.test_case_tree_bulk_controller_v2_api import TestCaseTreeBulkControllerV2Api
from src.client.generated.api.test_case_tree_controller_v2_api import TestCaseTreeControllerV2Api
from src.client.generated.api.test_layer_controller_api import TestLayerControllerApi
from src.client.generated.api.test_layer_schema_controller_api import TestLayerSchemaControllerApi
from src.client.generated.api.test_plan_controller_api import TestPlanControllerApi
from src.client.generated.api.tree_controller_v2_api import TreeControllerV2Api




"""Unit tests for CustomFieldValueService."""

from unittest.mock import AsyncMock, MagicMock

import pytest

from src.client import AllureClient
from src.client.exceptions import AllureValidationError
from src.client.generated.models.custom_field_value_with_cf_dto import CustomFieldValueWithCfDto
from src.client.generated.models.custom_field_value_with_tc_count_dto import CustomFieldValueWithTcCountDto
from src.client.generated.models.page_custom_field_value_with_tc_count_dto import PageCustomFieldValueWithTcCountDto
from src.services.custom_field_value_service import CustomFieldValueService


@pytest.fixture
def mock_client() -> MagicMock:
    client = MagicMock(spec=AllureClient)
    client.get_project.return_value = 1
    client.list_custom_field_values = AsyncMock()
    client.create_custom_field_value = AsyncMock()
    client.update_custom_field_value = AsyncMock()
    client.delete_custom_field_value = AsyncMock()
    return client


@pytest.fixture
def service(mock_client: MagicMock) -> CustomFieldValueService:
    return CustomFieldValueService(client=mock_client)


@pytest.mark.asyncio
async def test_list_custom_field_values_success(service: CustomFieldValueService, mock_client: MagicMock) -> None:
    page = PageCustomFieldValueWithTcCountDto(
        content=[CustomFieldValueWithTcCountDto(id=1, name="Smoke", test_cases_count=3)],
        total_elements=1,
        number=0,
        total_pages=1,
    )
    mock_client.list_custom_field_values.return_value = page
    service._get_resolved_custom_fields = AsyncMock(
        return_value={
            "Priority": {
                "project_cf_id": 10,
                "id": 5,
                "required": False,
                "single_select": True,
                "values": [],
                "values_map": {},
            },
        }
    )

    result = await service.list_custom_field_values(
        custom_field_name="Priority",
        query="Smoke",
        var_global=True,
        test_case_search="smoke",
        page=1,
        size=5,
        sort=["name,asc"],
    )

    assert result.total_elements == 1
    mock_client.list_custom_field_values.assert_called_once_with(
        project_id=1,
        custom_field_id=10,
        query="Smoke",
        var_global=True,
        test_case_search="smoke",
        page=1,
        size=5,
        sort=["name,asc"],
    )


@pytest.mark.asyncio
async def test_list_custom_field_values_invalid_name(service: CustomFieldValueService) -> None:
    service._get_resolved_custom_fields = AsyncMock(
        return_value={
            "Priority": {
                "project_cf_id": 10,
                "id": 5,
                "required": False,
                "single_select": True,
                "values": [],
                "values_map": {},
            },
        }
    )

    with pytest.raises(AllureValidationError, match="Custom field 'Missing' not found"):
        await service.list_custom_field_values(custom_field_name="Missing")


@pytest.mark.asyncio
async def test_list_custom_field_values_invalid_custom_field_id(service: CustomFieldValueService) -> None:
    with pytest.raises(AllureValidationError, match="Custom Field ID must be a non-zero integer"):
        await service.list_custom_field_values(custom_field_id=0)


@pytest.mark.asyncio
async def test_list_custom_field_values_accepts_negative_custom_field_id(
    service: CustomFieldValueService, mock_client: MagicMock
) -> None:
    page = PageCustomFieldValueWithTcCountDto(content=[], total_elements=0)
    mock_client.list_custom_field_values.return_value = page

    result = await service.list_custom_field_values(custom_field_id=-5)

    assert result.total_elements == 0
    mock_client.list_custom_field_values.assert_called_once_with(
        project_id=1,
        custom_field_id=-5,
        query=None,
        var_global=None,
        test_case_search=None,
        page=None,
        size=None,
        sort=None,
    )


@pytest.mark.asyncio
async def test_create_custom_field_value_success(service: CustomFieldValueService, mock_client: MagicMock) -> None:
    created = CustomFieldValueWithCfDto(id=99, name="Regression")
    mock_client.create_custom_field_value.return_value = created
    service._get_resolved_custom_fields = AsyncMock(
        return_value={
            "Priority": {
                "project_cf_id": 10,
                "id": 5,
                "required": False,
                "single_select": True,
                "values": [],
                "values_map": {},
            },
        }
    )
    service._test_case_service.refresh_resolved_custom_fields = AsyncMock()

    result = await service.create_custom_field_value(custom_field_name="Priority", name="Regression")

    assert result.id == 99
    mock_client.create_custom_field_value.assert_called_once()
    service._test_case_service.refresh_resolved_custom_fields.assert_awaited_once_with(1)


@pytest.mark.asyncio
async def test_create_custom_field_value_duplicate_name(
    service: CustomFieldValueService, mock_client: MagicMock
) -> None:
    service._get_resolved_custom_fields = AsyncMock(
        return_value={
            "Priority": {
                "project_cf_id": 10,
                "id": 5,
                "required": False,
                "single_select": True,
                "values": [],
                "values_map": {},
            },
        }
    )
    mock_client.create_custom_field_value.side_effect = AllureValidationError("Validation error", status_code=409)

    with pytest.raises(AllureValidationError, match="already exists"):
        await service.create_custom_field_value(custom_field_name="Priority", name="Regression")


@pytest.mark.asyncio
async def test_update_custom_field_value_success_no_usage(
    service: CustomFieldValueService, mock_client: MagicMock
) -> None:
    service._get_resolved_custom_fields = AsyncMock(
        return_value={
            "Priority": {"project_cf_id": 10, "id": 5},
        }
    )
    # Mock finding the value with 0 usage
    page = PageCustomFieldValueWithTcCountDto(
        content=[CustomFieldValueWithTcCountDto(id=77, name="Old", test_cases_count=0)], total_elements=1
    )
    mock_client.list_custom_field_values.return_value = page
    service._test_case_service.refresh_resolved_custom_fields = AsyncMock()

    await service.update_custom_field_value(custom_field_name="Priority", cfv_id=77, name="Updated")

    mock_client.update_custom_field_value.assert_called_once()
    service._test_case_service.refresh_resolved_custom_fields.assert_awaited_once_with(1)


@pytest.mark.asyncio
async def test_update_custom_field_value_fails_with_usage(
    service: CustomFieldValueService, mock_client: MagicMock
) -> None:
    service._get_resolved_custom_fields = AsyncMock(
        return_value={
            "Priority": {"project_cf_id": 10, "id": 5},
        }
    )
    # Mock finding the value with usage
    page = PageCustomFieldValueWithTcCountDto(
        content=[CustomFieldValueWithTcCountDto(id=77, name="Old", test_cases_count=5)], total_elements=1
    )
    mock_client.list_custom_field_values.return_value = page

    with pytest.raises(AllureValidationError, match="used in 5 test cases"):
        await service.update_custom_field_value(custom_field_name="Priority", cfv_id=77, name="Updated")

    mock_client.update_custom_field_value.assert_not_called()


@pytest.mark.asyncio
async def test_update_custom_field_value_success_with_force(
    service: CustomFieldValueService, mock_client: MagicMock
) -> None:
    service._get_resolved_custom_fields = AsyncMock(
        return_value={
            "Priority": {"project_cf_id": 10, "id": 5},
        }
    )
    # Mock finding the value with usage
    page = PageCustomFieldValueWithTcCountDto(
        content=[CustomFieldValueWithTcCountDto(id=77, name="Old", test_cases_count=5)], total_elements=1
    )
    mock_client.list_custom_field_values.return_value = page
    service._test_case_service.refresh_resolved_custom_fields = AsyncMock()

    await service.update_custom_field_value(custom_field_name="Priority", cfv_id=77, name="Updated", force=True)

    mock_client.update_custom_field_value.assert_called_once()


@pytest.mark.asyncio
async def test_delete_custom_field_value_by_name_success(
    service: CustomFieldValueService, mock_client: MagicMock
) -> None:
    service._get_resolved_custom_fields = AsyncMock(
        return_value={
            "Priority": {"project_cf_id": 10, "id": 5},
        }
    )
    # Mock finding the value by name
    page = PageCustomFieldValueWithTcCountDto(
        content=[CustomFieldValueWithTcCountDto(id=55, name="Target", test_cases_count=0)], total_elements=1
    )
    mock_client.list_custom_field_values.return_value = page
    service._test_case_service.refresh_resolved_custom_fields = AsyncMock()

    deleted = await service.delete_custom_field_value(custom_field_name="Priority", cfv_name="Target")

    assert deleted is True
    mock_client.delete_custom_field_value.assert_called_once_with(1, 55)
    service._test_case_service.refresh_resolved_custom_fields.assert_awaited_once_with(1)

    # Verify list called with query="Target"
    call_args = mock_client.list_custom_field_values.call_args
    assert call_args.kwargs["query"] == "Target"


@pytest.mark.asyncio
async def test_delete_custom_field_value_fails_with_usage(
    service: CustomFieldValueService, mock_client: MagicMock
) -> None:
    service._get_resolved_custom_fields = AsyncMock(
        return_value={
            "Priority": {"project_cf_id": 10, "id": 5},
        }
    )
    # Mock finding the value with usage
    page = PageCustomFieldValueWithTcCountDto(
        content=[CustomFieldValueWithTcCountDto(id=55, name="Target", test_cases_count=2)], total_elements=1
    )
    mock_client.list_custom_field_values.return_value = page

    with pytest.raises(AllureValidationError, match="used in 2 test cases"):
        await service.delete_custom_field_value(custom_field_name="Priority", cfv_name="Target")

    mock_client.delete_custom_field_value.assert_not_called()


@pytest.mark.asyncio
async def test_delete_custom_field_value_idempotent(service: CustomFieldValueService, mock_client: MagicMock) -> None:
    service._get_resolved_custom_fields = AsyncMock(
        return_value={
            "Priority": {"project_cf_id": 10, "id": 5},
        }
    )
    service._test_case_service.refresh_resolved_custom_fields = AsyncMock()

    # Mock NOT finding the value when resolving usage logic calls list
    page = PageCustomFieldValueWithTcCountDto(content=[], total_elements=0)
    mock_client.list_custom_field_values.return_value = page

    deleted = await service.delete_custom_field_value(custom_field_name="Priority", cfv_name="Missing")

    assert deleted is False
    mock_client.delete_custom_field_value.assert_not_called()
    service._test_case_service.refresh_resolved_custom_fields.assert_not_awaited()



# coding: utf-8

"""
    allure-testops-service

    Branch **HEAD**  Commit **623f6ed302ba4b651cf9040faca4635af2d93b7c**

    The version of the OpenAPI document: 25.4.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import StrictInt
from typing import List
from src.client.generated.models.test_tag_dto import TestTagDto

from src.client.generated.api_client import ApiClient, RequestSerialized
from src.client.generated.api_response import ApiResponse
from src.client.generated.rest import RESTResponseType


class TestCaseTagControllerApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    async def get_tags(
        self,
        test_case_id: StrictInt,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[TestTagDto]:
        """Find tags for test case


        :param test_case_id: (required)
        :type test_case_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_tags_serialize(
            test_case_id=test_case_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[TestTagDto]",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    async def get_tags_with_http_info(
        self,
        test_case_id: StrictInt,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[TestTagDto]]:
        """Find tags for test case


        :param test_case_id: (required)
        :type test_case_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_tags_serialize(
            test_case_id=test_case_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[TestTagDto]",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    async def get_tags_without_preload_content(
        self,
        test_case_id: StrictInt,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Find tags for test case


        :param test_case_id: (required)
        :type test_case_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_tags_serialize(
            test_case_id=test_case_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[TestTagDto]",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_tags_serialize(
        self,
        test_case_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if test_case_id is not None:
            _path_params['testCaseId'] = test_case_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    '*/*'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/testcase/{testCaseId}/tag',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    async def set_tags(
        self,
        test_case_id: StrictInt,
        test_tag_dto: List[TestTagDto],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[TestTagDto]:
        """Set test tags for test case


        :param test_case_id: (required)
        :type test_case_id: int
        :param test_tag_dto: (required)
        :type test_tag_dto: List[TestTagDto]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._set_tags_serialize(
            test_case_id=test_case_id,
            test_tag_dto=test_tag_dto,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[TestTagDto]",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    async def set_tags_with_http_info(
        self,
        test_case_id: StrictInt,
        test_tag_dto: List[TestTagDto],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[TestTagDto]]:
        """Set test tags for test case


        :param test_case_id: (required)
        :type test_case_id: int
        :param test_tag_dto: (required)
        :type test_tag_dto: List[TestTagDto]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._set_tags_serialize(
            test_case_id=test_case_id,
            test_tag_dto=test_tag_dto,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[TestTagDto]",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    async def set_tags_without_preload_content(
        self,
        test_case_id: StrictInt,
        test_tag_dto: List[TestTagDto],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Set test tags for test case


        :param test_case_id: (required)
        :type test_case_id: int
        :param test_tag_dto: (required)
        :type test_tag_dto: List[TestTagDto]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._set_tags_serialize(
            test_case_id=test_case_id,
            test_tag_dto=test_tag_dto,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[TestTagDto]",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _set_tags_serialize(
        self,
        test_case_id,
        test_tag_dto,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'TestTagDto': '',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if test_case_id is not None:
            _path_params['testCaseId'] = test_case_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if test_tag_dto is not None:
            _body_params = test_tag_dto


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    '*/*'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/testcase/{testCaseId}/tag',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# coding: utf-8

"""
    allure-testops-service

    Branch **HEAD**  Commit **623f6ed302ba4b651cf9040faca4635af2d93b7c**

    The version of the OpenAPI document: 25.4.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from importlib import import_module
from pydantic import BaseModel, ConfigDict, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from src.client.generated.models.node_type import NodeType
from typing import Optional, Set
from typing_extensions import Self

from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from src.client.generated.models.test_case_light_tree_node_dto import TestCaseLightTreeNodeDto
    from src.client.generated.models.test_case_tree_leaf_dto_v2 import TestCaseTreeLeafDtoV2

class TestCaseTreeNodeDto(BaseModel):
    """
    TestCaseTreeNodeDto
    """ # noqa: E501
    id: Optional[StrictInt] = None
    name: Optional[StrictStr] = None
    type: Optional[NodeType] = None
    __properties: ClassVar[List[str]] = ["id", "name", "type"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    # JSON field name that stores the object type
    __discriminator_property_name: ClassVar[str] = 'type'

    # discriminator mappings
    __discriminator_value_class_map: ClassVar[Dict[str, str]] = {
        'TestCaseLightTreeNodeDto': 'TestCaseLightTreeNodeDto','TestCaseTreeLeafDtoV2': 'TestCaseTreeLeafDtoV2'
    }

    @classmethod
    def get_discriminator_value(cls, obj: Dict[str, Any]) -> Optional[str]:
        """Returns the discriminator value (object type) of the data"""
        discriminator_value = obj[cls.__discriminator_property_name]
        if discriminator_value:
            return cls.__discriminator_value_class_map.get(discriminator_value)
        else:
            return None

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Union[TestCaseLightTreeNodeDto, TestCaseTreeLeafDtoV2]]:
        """Create an instance of TestCaseTreeNodeDto from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict[str, Any]) -> Optional[Union[TestCaseLightTreeNodeDto, TestCaseTreeLeafDtoV2]]:
        """Create an instance of TestCaseTreeNodeDto from a dict"""
        # look up the object type based on discriminator mapping
        object_type = cls.get_discriminator_value(obj)
        if object_type ==  'TestCaseLightTreeNodeDto':
            return import_module("src.client.generated.models.test_case_light_tree_node_dto").TestCaseLightTreeNodeDto.from_dict(obj)
        if object_type ==  'TestCaseTreeLeafDtoV2':
            return import_module("src.client.generated.models.test_case_tree_leaf_dto_v2").TestCaseTreeLeafDtoV2.from_dict(obj)

        raise ValueError("TestCaseTreeNodeDto failed to lookup discriminator value from " +
                            json.dumps(obj) + ". Discriminator property name: " + cls.__discriminator_property_name +
                            ", mapping: " + json.dumps(cls.__discriminator_value_class_map))


